﻿--[[-----------------------------------------------------------------------------
Name: Soar-1.0
Revision: $Revision: 44445 $
Author(s): Antiarc (cheald at gmail)
Website: http://www.wowace.com/wiki/Soar-1.0
Documentation: http://www.wowace.com/wiki/Soar-1.0
SVN: http://svn.wowace.com/wowace/trunk/Soar-1.0/
Description: Abstracts the process of sending notification to floating combat text addons
License: LGPL v2.1

Copyright (C) 2007 Chris Heald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-------------------------------------------------------------------------------]]

local MAJOR_VERSION = "Soar-1.0"
local MINOR_VERSION = tonumber(("$Revision: 44445 $"):match("%d+"))

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

local SML = AceLibrary("SharedMedia-1.0")

local AceOO = AceLibrary:GetInstance("AceOO-2.0")

local soar = AceOO.Mixin {
	"SoarMessage", "SoarNotify", "SoarIncoming", "SoarOutgoing", "RegisterSoarHandler"
}

-- #NODOC
function soar:Parrot(text, r, g, b, font, size, outline, sticky, location)
	local location = (location == "INCOMING" and "Incoming") or (location == "OUTGOING" and "Incoming") or "Notification"
	Parrot:ShowMessage(text, location, sticky, r, g, b, font, size, outline)
end

-- #NODOC
local sct_color = {}
function soar:SCT(text, r, g, b, font, size, outline, sticky, location)
	sct_color.r, sct_color.g, sct_color.b = r, g, b
	local location = (location == "INCOMING" and SCT.FRAME1) or (location == "OUTGOING" and SCT.FRAME2) or SCT.MSG
	SCT:DisplayCustomEvent(text, sct_color, sticky, location)
end

-- #NODOC
local msbt_registered_fonts = {}
local msbt_outlines = {["NORMAL"] = 1, ["OUTLINE"] = 2, ["THICKOUTLINE"] = 3} 
function soar:MSBT(text, r, g, b, font, size, outline, sticky, location)
	if not msbt_registered_fonts[font] and font then
		MikSBT.RegisterFont(font, SML:Fetch("font", font))
		msbt_registered_fonts[font] = true
	end
	local location = (location == "INCOMING" and MikSBT.DISPLAYTYPE_INCOMING) or 
		(location == "OUTGOING" and MikSBT.DISPLAYTYPE_OUTGOING) or 
		MikSBT.DISPLAYTYPE_NOTIFICATION

	MikSBT.DisplayMessage(text, location, sticky, r * 255, g * 255, b * 255, size, font, msbt_outlines[outline] or 1)
end

-- #NODOC
function soar:Blizzard(text, r, g, b, font, size, outline, sticky, location)
	if tostring(SHOW_COMBAT_TEXT) ~= "0" then
		CombatText_AddMessage(text, CombatText_StandardScroll, r, g, b, sticky and "crit" or nil, false)
	else
		UIErrorsFrame:AddMessage("|cff" .. ("%02x%02x%02x"):format(r * 255, g * 255, b * 255) .. text .. "|r")
	end
end

local handler = nil
local customHandlers = {
	["Parrot"] = soar.Parrot,
	["SCT"] = soar.SCT,
	["MikSBT"] = soar.MSBT
}

local function GetAddOnHandler()
	if handler then return handler end
	for name, callback in pairs(customHandlers) do
		if _G[name] then
			handler = callback
			return handler
		end
	end
	handler = soar.Blizzard
	return handler
end

--[[------------------------------------------------------------
Arguments:
	string - The name of the global table for the addon we wish to check the existance of.
	callback - function to call to show text if the related addon is found. Callback must have the same signature as :ShowSoarMessage
Returns:
	None
Example:
--	function foo(text, r, g, b, font, size, outline, sticky)
--	end
--	
--	self:RegisterSoarHandler("MyAddon", foo)
--------------------------------------------------------------]]
function soar:RegisterSoarHandler(addonName, callback)
	customHandlers[addonName] = callback
	handler = nil
end

--[[------------------------------------------------------------
Arguments:
	string - text to show
	[optional] int - red color value [0-1]
	[optional] int - green color value [0-1]
	[optional] int - blue color value [0-1]
	[optional] string - "NORMAL", "OUTLINE", or "THICKOUTLINE"
	[optional] boolean - indicate whether this should be a "sticky"/crit event
	[optional] string - "NOTIFY", "INCOMING", or "OUTGOING". Default is "NOTIFY"
Returns:
	none
Notes:
	Sends a floating message to the first supported addon found. If no location is specified, the message is sent to the notification frame (if available)
Example:	
	-- Sends a yellow message in the default font with a thick outline to the notification frame
	self:SoarMessage("Hello from my addon!", 1, 1, 0, nil, nil, "THICKOUTLINE", true, "NOTIFY")
--------------------------------------------------------------]]
function soar:SoarMessage(text, r, g, b, font, size, outline, sticky, location)
	GetAddOnHandler()(soar, text, r or 1, g or 1, b or 1, font or nil, size or nil, outline or nil, sticky or false, location)
end

--[[------------------------------------------------------------
Notes:
	An alias for :SoarMessage() with a location of "NOTIFY"
Example:	
	-- Sends a yellow message in the default font with a thick outline to the notification frame
	self:SoarNotify("Hello from my addon!")
--------------------------------------------------------------]]
function soar:SoarNotify(text, r, g, b, font, size, outline, sticky)
	GetAddOnHandler()(soar, text, r or 1, g or 1, b or 1, font or nil, size or nil, outline or nil, sticky or false, "NOTIFY")
end

--[[------------------------------------------------------------
Notes:
	An alias for :SoarMessage() with a location of "INCOMING"
Example:	
	-- Sends a yellow message in the default font with a thick outline to the incoming text frame
	self:SoarIncoming("Hello from my addon!")
--------------------------------------------------------------]]
function soar:SoarIncoming(text, r, g, b, font, size, outline, sticky)
	GetAddOnHandler()(soar, text, r or 1, g or 1, b or 1, font or nil, size or nil, outline or nil, sticky or false, "INCOMING")
end

--[[------------------------------------------------------------
Notes:
	An alias for :SoarMessage() with a location of "OUTGOING"
Example:	
	-- Sends a yellow message in the default font with a thick outline to the outgoing text frame
	self:SoarOutgoing("Hello from my addon!")
--------------------------------------------------------------]]
function soar:SoarOutgoing(text, r, g, b, font, size, outline, sticky)
	GetAddOnHandler()(soar, text, r or 1, g or 1, b or 1, font or nil, size or nil, outline or nil, sticky or false, "OUTGOING")
end

local function activate(self, oldLib, oldDeactivate)
	soar = self
end

AceLibrary:Register(soar, MAJOR_VERSION, MINOR_VERSION, activate) 
